import { PermissionFlagsBits, InteractionType } from 'discord.js';
import db from '../../models/index.js';

const { ownerId } = process.env;

export default {
	name: 'interactionCreate',
	async execute(interaction) {
		const ratelimit = global.ratelimit;
		const client = interaction.client;
		if (interaction.type !== InteractionType.ApplicationCommand) return;

		const globalBlacklist = await db.Blacklists.findOne({ where: { type:'global', uid:interaction.user.id } });
		const commandBlacklist = await db.Blacklists.findOne({ where: { type:interaction.commandName, uid:interaction.user.id } });
		if (globalBlacklist) {
			return interaction.reply({ content: `You are globally blacklisted for the following reason: \`${globalBlacklist.reason}\``, ephemeral: true });
		}
		else if (commandBlacklist) {
			return interaction.reply({ content: `You are blacklisted for the following reason: \`${commandBlacklist.reason}\``, ephemeral: true });
		}

		const userTag = interaction.user.tag;
		const userID = interaction.user.id;
		const commandName = interaction.commandName;

		const command = client.commands.get(commandName);

		if (!command) return;

		console.log(`\x1b[33m${userTag} (${userID})\x1b[0m launched command \x1b[33m${commandName}\x1b[0m`);

		// Owner only check
		if (command.ownerOnly && interaction.user.id !== ownerId) {
			return interaction.reply({ content: '❌ This command is reserved for the owner!', ephemeral: true });
		}

		// Check if the bot has the needed permissions
		if (command.default_permission) {
			const clientMember = await interaction.guild.members.fetch(client.user.id);
			if (!clientMember.permissions.has(command.clientPermissions)) {
				return interaction.reply({ content: `❌ I am missing one of the following permission(s): \`${new PermissionFlagsBits(command.clientPermissions).toArray()}\``, ephemeral: true });
			}
		}

		// Check if the user has the needed permissions
		/*
		if (command.default_member_permissions) {
			if (!interaction.member.permissions.has(command.userPermissions)) {
				return interaction.reply({ content: `❌ You are missing one of the following permission(s): \`${new PermissionFlagsBits(command.userPermissions).toArray()}\``, ephemeral: true });
			}
		}
		*/

		try {
			if (!ratelimit[userID]) {
				ratelimit[userID] = {};
			}

			const date = new Date();
			if (ratelimit[userID][commandName]) {
				if (ratelimit[userID][commandName].cooldown) {
					if (date > ratelimit[userID][commandName].cooldown) {
						ratelimit[userID][commandName].limit = 0;
						ratelimit[userID][commandName].cooldown = undefined;
					}
				}

				if (command.ratelimit === ratelimit[userID][commandName].limit) {
					return await interaction.reply({ content: `You are being rate limited. You can try again in ${Math.floor((ratelimit[userID][commandName].cooldown - date) / 1000)} seconds.`, ephemeral: true });
				}
			}


			if (command.ratelimit) {
				ratelimit[userID][commandName] = { limit: ratelimit[userID][commandName] ? ratelimit[userID][commandName].limit + 1 : 1 };
				if (command.ratelimit === ratelimit[userID][commandName].limit) {
					date.setSeconds(date.getSeconds() + command.cooldown);

					ratelimit[userID][commandName] = { limit: ratelimit[userID][commandName].limit, cooldown: date };
				}
			}

			interaction.prefix = '/';

			const args = {};
			// https://discord-api-types.dev/api/discord-api-types-v10/enum/ApplicationCommandOptionType
			interaction.options.data.forEach(arg => {
				let payload = arg.value;
				if (arg.type === 9) {
					payload = arg.member;
				}
				else if (arg.type === 11) {
					payload = arg.attachment;
				}
				args[arg.name] = payload;
			});

			await command.execute(interaction, args, client);
		}
		catch (error) {
			console.error(error);
			await interaction.followUp({ content: 'There was an error while executing this command!', ephemeral: true });
		}
	},
};
